from .. import messages as msg
from .run import run_command


def get_git_version(short=False):
    res, _ = run_command(['git', '--version'], print_stderr=False)
    return res.stdout.strip().split()[2], _


def get_gh_version(short=False):
    res, _ = run_command(['gh', '--version'], print_stderr=False)
    short_v = res.stdout.strip().split()[-3]
    long_v = res.stdout.strip().split('\n')[0].split('version ')[-1]
    if short:
        return short_v, _
    return long_v, _


def get_uv_version(short=False):
    res, _ = run_command(['uv', 'self', 'version', '--short', '--color', 'never'], print_stderr=False)
    if short:
        return res.stdout.strip().split()[0], _
    return res.stdout.strip(), _


def get_uv_ship_version(short=False):
    res, _ = run_command(['uv-ship', '--self'], print_stderr=False)
    if short:
        return res.stdout.strip().split()[-1], _
    return res.stdout.strip(), _


def get_self_version(short=False):
    from importlib.metadata import version

    return version('uv_ship'), True


def get_tool_versions(print_status=False):
    versions = {
        'uv': {'func': get_uv_version, 'required': True},
        'uv-ship': {'func': get_self_version, 'required': True},
        'git': {'func': get_git_version, 'required': True},
        'github': {'func': get_gh_version, 'required': False},
    }

    results = {}
    for k, param in versions.items():
        try:
            version, _ = param['func'](short=True)
        except Exception:
            version, _ = None, False

        if not _ and param['required']:
            if print_status:
                msg.failure(f'{k} is not installed or not available on PATH.')
            # version = False
        elif not _ and not param['required']:
            if print_status:
                msg.warning(f'{k} is not installed or not available on PATH.')
            results[k] = 'not installed'
        elif _ and param['required']:
            if print_status:
                msg.success(f'{k} is installed and available on PATH.')
            results[k] = version

    return results



from .resources import ac, sym


def imsg(text: str, icon=None, color=None, **kwargs):
    icon = '' if icon is None else f'{icon} '
    color = ac.RESET if color is None else color
    print(f'{color}{icon}{text}{ac.RESET}', **kwargs)


def failure(message):
    imsg(f'{message}\n', icon=sym.negative, color=ac.RED)
    exit(1)


def warning(message):
    imsg(f'{message}', icon=sym.warning, color=ac.YELLOW)


def success(message):
    imsg(f'{message}', icon=sym.positive, color=ac.GREEN)


def abort_by_user():
    failure('aborted by user.')


def dry_run_warning():
    imsg('>> THIS IS A DRY RUN - NO CHANGES WILL BE MADE <<\n', color=ac.DIM)


def welcome_message():
    print('')
    imsg(f'{ac.GREEN}uv-ship', color=ac.BOLD, end=' - ')
    imsg('a CLI-tool for shipping with uv', color=None)


def preflight_complete():
    print('')
    imsg('have you updated the documentation?', icon=None, color=ac.BLUE)

    # all preflight checks passed
    print('')
    imsg(f'{"-" * 62}', icon=None)
    imsg('If everything looks good, you can proceed with the release.', icon=None)
    # msg.imsg('ready to ship!', icon=sym.positive)

    step_by_step_operations()


def step_by_step_operations():
    operations_message = [
        '',
        'the following operations will be performed:',
        '  1. update version in pyproject.toml and uv.lock',
        '  2. create a tagged commit with the updated files',
        '  3. push changes to the remote repository\n',
    ]
    print('\n'.join(operations_message))


def user_confirmation():
    confirm = input('do you want to proceed? [y/N]: ').strip().lower()
    if confirm not in ('y', 'yes'):
        abort_by_user()



from . import commands as cmd
from . import config as cfg
from . import messages as msg
from . import workflows as wfl
from .resources import rich_click as click

click.rich_click.COMMAND_GROUPS = {
    'uv-ship': [
        {'name': 'commands', 'commands': ['next', 'version', 'log', 'status']},
        # {"name": "utilities", "commands": ["log"]},
    ],
}


# region cli
@click.group(
    context_settings=dict(help_option_names=['-h', '--help']), invoke_without_command=True, add_help_option=True
)
# @click.group(invoke_without_command=True)
@click.option('--config', type=click.Path(exists=True), help='Path to config file (inferred if not provided).')
@click.option('--dry-run', is_flag=True, default=False, help='Review changes without touching disk.')
@click.option('--self', is_flag=True, default=False, help='Display uv-ship version.')
@click.pass_context
def cli(ctx, dry_run, config, self):
    if self:
        ver = cmd.ver.get_self_version(short=True)
        click.echo(f'uv-ship {ver[0]}')
    else:
        # Show tagline and set up config
        msg.welcome_message()

        repo_root = cmd.git.get_repo_root()
        uvs_config = cfg.load_config(path=config, cwd=repo_root, cmd_args={'dry_run': dry_run})

        uv_version, _ = cmd.run_command(['uv', 'self', 'version', '--short'], print_stderr=False)
        if not _:
            msg.failure('uv is not installed or not available on PATH.')
        else:
            msg.imsg(f'uv version {uv_version.stdout.split()[0]}', color=msg.ac.DIM)
        # print('')

        if uvs_config['dry_run']:
            msg.dry_run_warning()

        # store config in context so subcommands can use it
        ctx.ensure_object(dict)
        ctx.obj = uvs_config

        # No subcommand given → show help
        if ctx.invoked_subcommand is None:
            click.echo(ctx.get_help())
            # print('')
            ctx.exit()


# region next
@cli.command(name='next')
@click.argument('release-type', type=str, panel='commands')
@click.option('--pre-release', type=str, default=None, help='Pre-release component (e.g. alpha, beta).')
@click.option('--dirty', is_flag=True, default=None, help='Allow dirty working directory.')
@click.pass_context
def cli_next(ctx, release_type, pre_release, dirty):
    """
    bump and ship the next project version.

    \b
    release_types:                major, minor, patch, *stable
    pair with pre-release:        alpha, beta, rc, post, dev (optional)
    \b
    remove pre-release status:    set release_type to 'stable'
    """
    # show summary
    next_step = release_type if not pre_release else f'{release_type} ({pre_release})'
    msg.imsg(f'bumping to the next {next_step} version:', color=msg.ac.BLUE)
    version = cmd.gen.calculate_version(bump_type=release_type, pre_release=pre_release)
    wfl.ship(config=ctx.obj, version=version, allow_dirty=dirty)


# region version
@cli.command(name='version')
@click.argument('version', type=str, panel='commands')
@click.option('--dirty', is_flag=True, default=None, help='Allow dirty working directory.')
@click.pass_context
def cli_version(ctx, version, dirty):
    """
    set, tag, and ship a specific version.
    """
    msg.imsg('setting a new project version:', color=msg.ac.BLUE)
    wfl.ship(config=ctx.obj, version=version, allow_dirty=dirty)


# region log
@cli.command(name='log')
@click.option('--tag', type=str, default='latest', help='Tag to use in the changelog.')
@click.option('--latest', is_flag=True, help='Show all commits since the last tag.')
@click.option('--save', is_flag=True, default=None, help='Save changes to the changelog.')
@click.pass_context
def log(ctx, tag, latest, save):
    """
    build/show the changelog.
    """
    wfl.cmd_log(config=ctx.obj, new_tag=tag, latest=latest, save=save)


# region log
@cli.command(name='status')
@click.pass_context
def status(ctx):
    """
    show project status.
    """
    wfl.cmd_status(config=ctx.obj)


if __name__ == '__main__':
    cli(prog_name='uv-ship')



from .. import messages as msg
from ..resources import sym
from .run import run_command


def get_version_str(return_project_name: bool = False):
    result, _ = run_command(['uv', 'version', '--color', 'never'])
    project_name, version = result.stdout.strip().split(' ')

    if return_project_name:
        return project_name, version

    return version


def collect_info(version: str = None):
    result, _ = run_command(['uv', 'version', version, '--dry-run', '--color', 'never'], print_stderr=False)
    if not _:
        msg.failure(result.stderr.strip())

    package_name, old_version, _, new_version = result.stdout.strip().split(' ')
    return package_name, old_version, new_version


def calculate_version(bump_type: str, pre_release: str = None):
    possible_bumps = ['major', 'minor', 'patch', 'stable', 'alpha', 'beta', 'rc', 'post', 'dev']
    if bump_type not in possible_bumps:
        msg.failure(f'invalid release type: "{bump_type}"\n  possible values: {", ".join(possible_bumps)}')

    command = ['uv', 'version', '--dry-run', '--color', 'never', '--bump', bump_type]
    command = command if not pre_release else command + ['--bump', pre_release]
    res, _ = run_command(command, print_stderr=False)

    return res.stdout.strip().split(' ')[-1]


def tag(config: dict, new_ver: str):
    tag_prefix = config['tag_prefix']
    return f'{tag_prefix}{new_ver}'


def commit_message(config: dict, old_ver: str, new_ver: str):
    return config['commit_message'].format(old_ver=old_ver, new_ver=new_ver)


def update_files(config, package_name, version):
    msg.imsg(f'updating {package_name} version', icon=sym.item)

    if not config['dry_run']:
        _, success = run_command(['uv', 'version', version])
        exit(1) if not success else None


# region unused
def pre_commit_checks():
    msg.imsg('running pre-commit checks', icon=sym.item)
    _, success = run_command(['pre-commit', 'run', '--all-files'], print_stdout=False)
    msg.failure('failed to run pre-commit checks') if not success else None



from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version('uv_ship')
except PackageNotFoundError:
    __version__ = 'unknown'

from . import changelogger as cl  # noqa: F401
from . import commands as cmd  # noqa: F401
from . import config as cfg  # noqa: F401
from . import messages as msg  # noqa: F401
